@@ -3,8 +3,7 @@ from __future__ import (
 
 from ufo2ft.filters import loadFilters
 from ufo2ft.filters.decomposeComponents import DecomposeComponentsFilter
-
-from copy import deepcopy
+from ufo2ft.util import copyGlyphSet
 
 
 class BasePreProcessor(object):
@@ -30,15 +29,16 @@ class BasePreProcessor(object):
 
     def __init__(self, ufo, inplace=False, **kwargs):
         self.ufo = ufo
+        self.inplace = inplace
         if inplace:
             self.glyphSet = {g.name: g for g in ufo}
         else:
-            self.glyphSet = {g.name: _copyGlyph(g) for g in ufo}
+            self.glyphSet = copyGlyphSet(ufo)
         self.defaultFilters = self.initDefaultFilters(**kwargs)
         self.preFilters, self.postFilters = loadFilters(ufo)
 
     def initDefaultFilters(self, **kwargs):
-        return []
+        return []  # pragma: no cover
 
     def process(self):
         ufo = self.ufo
@@ -88,10 +88,16 @@ class TTFPreProcessor(OTFPreProcessor):
     convention. Thus, by default the direction is reversed, in order to
     conform to opposite clockwise convention for TrueType outlines.
     You can disable this by setting ``reverseDirection`` to False.
+
+    If both ``inplace`` and ``rememberCurveType`` options are True, the curve
+    type "quadratic" is saved in font' lib under a private cu2qu key; the
+    preprocessor will not try to convert them again if the curve type is
+    already set to "quadratic".
     """
 
     def initDefaultFilters(self, removeOverlaps=False, convertCubics=True,
-                           conversionError=None, reverseDirection=True):
+                           conversionError=None, reverseDirection=True,
+                           rememberCurveType=True):
         # len(g) is the number of contours, so we include the all glyphs
         # that have both components and at least one contour
         filters = [DecomposeComponentsFilter(include=lambda g: len(g))]
@@ -102,10 +108,14 @@ class TTFPreProcessor(OTFPreProcessor):
 
         if convertCubics:
             from ufo2ft.filters.cubicToQuadratic import CubicToQuadraticFilter
-            filters.append(
-                CubicToQuadraticFilter(conversionError=conversionError,
-                                       reverseDirection=reverseDirection))
 
+            filters.append(
+                CubicToQuadraticFilter(
+                    conversionError=conversionError,
+                    reverseDirection=reverseDirection,
+                    rememberCurveType=rememberCurveType and self.inplace,
+                )
+            )
         return filters
 
 
@@ -120,46 +130,39 @@ class TTFInterpolatablePreProcessor(object):
     decomposition of mixed contour/component glyphs is performed,
     and no additional custom filter is applied.
 
-    The ``conversionError`` and ``reverseDirection`` arguments work in the
-    same way as in the ``TTFPreProcessor``.
+    The ``conversionError``, ``reverseDirection`` and ``rememberCurveType``
+    arguments work in the same way as in the ``TTFPreProcessor``.
     """
 
     def __init__(self, ufos, inplace=False, conversionError=None,
-                 reverseDirection=True):
+                 reverseDirection=True, rememberCurveType=True):
         from cu2qu.ufo import DEFAULT_MAX_ERR
 
         self.ufos = ufos
-        self.glyphSets = [
-            {g.name: (_copyGlyph(g) if not inplace else g) for g in ufo}
-            for ufo in ufos]
+        self.inplace = inplace
+        if inplace:
+            self.glyphSets = [{g.name: g for g in ufo} for ufo in ufos]
+        else:
+            self.glyphSets = [copyGlyphSet(ufo) for ufo in ufos]
         self._conversionErrors = [
             (conversionError or DEFAULT_MAX_ERR) * ufo.info.unitsPerEm
             for ufo in ufos]
         self._reverseDirection = reverseDirection
+        self._rememberCurveType = rememberCurveType
 
     def process(self):
         from cu2qu.ufo import fonts_to_quadratic
 
-        fonts_to_quadratic(self.glyphSets, max_err=self._conversionErrors,
-                           reverse_direction=self._reverseDirection,
-                           dump_stats=True)
+        fonts_to_quadratic(
+            self.ufos if self.inplace else self.glyphSets,
+            max_err=self._conversionErrors,
+            reverse_direction=self._reverseDirection,
+            dump_stats=True,
+            remember_curve_type=self._rememberCurveType and self.inplace,
+        )
 
         decompose = DecomposeComponentsFilter(include=lambda g: len(g))
         for ufo, glyphSet in zip(self.ufos, self.glyphSets):
             decompose(ufo, glyphSet)
 
         return self.glyphSets
-
-
-def _copyGlyph(glyph):
-    # copy everything except unused attributes: 'guidelines', 'note', 'image'
-    copy = glyph.__class__()
-    copy.name = glyph.name
-    copy.width = glyph.width
-    copy.height = glyph.height
-    copy.unicodes = list(glyph.unicodes)
-    copy.anchors = [dict(a) for a in glyph.anchors]
-    copy.lib = deepcopy(glyph.lib)
-    pointPen = copy.getPointPen()
-    glyph.drawPoints(pointPen)
-    return copy
